const { check, body, validationResult } = require("express-validator");
const express = require("express");
const peopleRouter = express.Router();
const {
  createPerson,
  findByName,
  deleteById,
  updatePersonById,
  getPersonFullDetails,
} = require("../service/peopleService");
/*
Function to check that the POST request body in 
create person contains the correct data types 
*/
const validateCreatePerson = [
  // Validate personInfo
  check("personInfo.firstName")
    .notEmpty()
    .withMessage("First name is required")
    .isString()
    .withMessage("First name must be a string"),

  check("personInfo.lastName")
    .notEmpty()
    .withMessage("Last name is required")
    .isString()
    .withMessage("Last name must be a string"),

  check("personInfo.classYear")
    .notEmpty()
    .withMessage("Class year is required")
    .isInt()
    .withMessage("Class year must be an integer"),

  // check if greater than 0
  check("personInfo.gradYear")
    .optional()
    .isInt()
    .withMessage("Grad year must be an integer if provided"),

  check("personInfo.gradSemester")
    .optional()
    .isString()
    .withMessage("Grad semester must be an String if provided"),

  check("personInfo.gender")
    .isIn(["male", "female", "other"])
    .withMessage("Invalid gender"),

  check("personInfo.maidenName")
    .optional()
    .isString()
    .withMessage("Maiden name should be a string"),

  // Validate degrees array
  body("degrees").isArray().withMessage("Degrees should be an array"),

  body("degrees.*.degreeType").isString().withMessage("Invalid degree type"),

  body("degrees.*.degreeYear")
    .notEmpty()
    .withMessage("Degree year is required")
    .isInt()
    .withMessage("Degree year must be an integer"),

  body("degrees.*.degreeDescription")
    .optional()
    .isLength({ max: 255 })
    .withMessage("Degree description cannot exceed 255 characters"),

  // Validate addresses array
  body("addresses").isArray().withMessage("Addresses should be an array"),

  body("addresses.*.address1")
    .notEmpty()
    .withMessage("Address line 1 is required"),

  body("addresses.*.city").notEmpty().withMessage("City is required"),

  body("addresses.*.state").notEmpty().withMessage("State is required"),

  body("addresses.*.country").notEmpty().withMessage("Country is required"),

  body("addresses.*.zipCode")
    .optional()
    .isInt()
    .withMessage("Zip code must be an integer if provided"),

  // Validate contacts array
  body("contacts").isArray().withMessage("Contacts should be an array"),

  body("contacts.*.contactNumber")
    .notEmpty()
    .withMessage("Contact number is required")
    .isString()
    .withMessage("Contact number must be a string"),

  body("contacts.*.contactType")
    .isIn(["phone", "email"])
    .withMessage("Invalid contact type"),

  body("contacts.*.preferredContact")
    .isBoolean()
    .withMessage("Preferred contact must be a boolean"),

  // Validate involvements
  body("involvements")
    .optional()
    .isArray()
    .withMessage("Involvements should be an array"),

  body("involvements.*")
    .optional()
    .isString()
    .withMessage("Each involvement must be a string"),
];

peopleRouter.post("/create", validateCreatePerson, async (req, res) => {
  // ensure the request body has the correct data types
  const errors = validationResult(req);

  // error found: return early
  if (!errors.isEmpty()) {
    return res.status(400).json({ errors: errors.array() });
  }

  try {
    const person = await createPerson(req.body);
    console.log("In people routes after add: ", person);

    return res
      .status(201)
      .json({ message: "person added successfully", personId: person });
  } catch (error) {
    console.log("error");
    return res.status(500).json({ message: error });
  }
});

/*
Route to search for people by first and last name. Request body takes
firstName and lastName as strings.
*/
peopleRouter.post("/searchByName", async (req, res) => {
  // get the first and last name
  const { firstName, lastName } = req.body;

  // Basic validation to ensure firstName and lastName are provided
  if (!firstName || !lastName) {
    return res
      .status(400)
      .json({ message: "First name and last name are required." });
  }

  try {
    // run function to get person by name
    const people = await findByName(firstName, lastName);

    // no person found
    if (people == null) {
      return res.status(404).json({ message: "person not found" });
    }

    // return the found people
    return res.status(200).json({ people: people });
  } catch (error) {
    // error
    console.log("error");
    return res.status(500).json({ message: error });
  }
});

// DELETE endpoint to delete a person by their ID
peopleRouter.delete("/:id", async (req, res) => {
  const personId = parseInt(req.params.id, 10); // Convert to an integer

  // Check if personId is a valid integer
  if (!Number.isInteger(personId)) {
    return res
      .status(400)
      .json({ message: "Invalid ID. ID must be an integer." });
  }

  try {
    // Call the function to delete the person
    const success = await deleteById(personId);

    if (!success) {
      // If no person is deleted, return a 404 status
      return res
        .status(404)
        .json({ message: "Person not found or could not be deleted." });
    }

    // TODO: log database change here

    // Return a success message
    res.json({ message: "Person successfully deleted." });
  } catch (error) {
    console.error("Error in DELETE /person/:id", error);
    res.status(500).json({ message: "Internal server error." });
  }
});

peopleRouter.put("/update/:id", async (req, res) => {
  // get the personId from the URL params
  const personId = parseInt(req.params.id, 10);

  // Check if personId is a valid integer
  if (!Number.isInteger(personId)) {
    return res
      .status(400)
      .json({ message: "Invalid ID. ID must be an integer." });
  }

  try {
    // Call the update service
    console.log("req.body: ", req.body);
    const success = await updatePersonById(personId, req.body);

    // no person was updated
    if (!success) {
      return res.status(404).json({ message: "Person not found" });
    }

    return res
      .status(200)
      .json({ message: "Person updated successfully", personId });
  } catch (error) {
    console.error("Error in PUT /update/:id", error);
    return res.status(500).json({ message: "Internal server error" });
  }
});

peopleRouter.get("/fullInfo/:id", async (req, res) => {
  // get the personId from the URL params
  const personId = parseInt(req.params.id, 10);

  // Check if personId is a valid integer
  if (!Number.isInteger(personId)) {
    return res
      .status(400)
      .json({ message: "Invalid ID. ID must be an integer." });
  }

  try {
    const person = await getPersonFullDetails(personId);

    if (person == null) {
      return res.status(404).json({ message: "Person not found" });
    }

    return res.status(200).json({ message: "Person found", person });
  } catch (error) {
    console.error("Error in GET /fullInfo/:id", error);
    return res.status(500).json({ message: "Internal server error" });
  }
});
module.exports = peopleRouter;
